from __future__ import absolute_import, division, print_function, with_statement

import pytest
from tornado import concurrent

import tornado.gen
import tornado.testing

import rw.scope


# def test_low_level_api():
#     scope = rw.scope.Scope()
#     current_user = object()
#
#     def get_current_user():
#         get_current_user.runs += 1
#         return current_user
#     scope.provider('user', get_current_user)
#     get_current_user.runs = 0
#
#     assert get_current_user.runs == 0
#     assert rw.scope.get('user') is current_user
#     assert get_current_user.runs == 1
#     # make sure provider is not run twice
#     rw.scope.get('user')
#     assert get_current_user.runs == 1
#
#     assert scope.get('unknown value', 'default') == 'default'
#     with pytest.raises(IndexError):
#         scope.get('unknown value')


def test_basic():
    scope = rw.scope.Scope()
    scope['some_static_value'] = 42
    current_user = object()

    def get_current_user():
        return current_user
    scope.provider('user', get_current_user)

    @rw.scope.inject
    def foo(user):
        return user

    @rw.scope.inject
    def bar(some_static_value):
        return some_static_value

    @rw.scope.inject
    def some_function_with_a_default_value(my_paramenter='my_default_value'):
        return my_paramenter

    with scope():
        assert foo() is current_user
        assert foo(1) == 1
        assert foo() is current_user
        assert bar() == 42
        assert bar(10) == 10
        assert bar(some_static_value=11) == 11

        # normal calling behaviour must be preserved
        assert some_function_with_a_default_value('value') == 'value'
        assert some_function_with_a_default_value() == 'my_default_value'

        # check nested scope
        nested_scope = rw.scope.Scope()
        nested_scope['user'] = 2
        with nested_scope():
            assert foo() == 2
            assert bar() == 42

        assert foo() is current_user
        assert bar() == 42


def test_recursion():
    """Entering the same scope twice should not produce unexpected behaviour"""
    scope = rw.scope.Scope()
    scope2 = rw.scope.Scope()

    with scope():
        assert rw.scope.get_current_scope() is scope
        with scope2():
            assert rw.scope.get_current_scope() is scope2
            with scope2():
                assert rw.scope.get_current_scope() is scope2
                with scope():
                    assert rw.scope.get_current_scope() is scope
                assert rw.scope.get_current_scope() is scope2
            assert rw.scope.get_current_scope() is scope2
        assert rw.scope.get_current_scope() is scope

    assert rw.scope.get_current_scope() is None


def test_sub_scope():
    scope1 = rw.scope.Scope()
    scope2 = rw.scope.Scope()
    scope3 = rw.scope.Scope()

    sub1 = scope1.subscope('my_sub_scope')
    sub2 = scope2.subscope('my_sub_scope')

    sub1['value_1'] = 1
    sub1['shared'] = 1
    sub2['value_2'] = 2
    sub2['shared'] = 2

    @rw.scope.inject
    def get_sub_scope(my_sub_scope):
        return my_sub_scope

    @rw.scope.inject
    def get_sub_scope_var(var, my_sub_scope):
        return my_sub_scope[var]

    def checks_inside_scope1():
        assert rw.scope.get('my_sub_scope') == get_sub_scope()
        assert rw.scope.get('my_sub_scope')['value_1'] == 1
        assert get_sub_scope_var('value_1') == 1
        assert rw.scope.get('my_sub_scope')['shared'] == 1
        assert get_sub_scope_var('shared') == 1
        assert 'value_2' not in rw.scope.get('my_sub_scope')

    def checks_inside_scope2():
        assert rw.scope.get('my_sub_scope') == get_sub_scope()
        assert rw.scope.get('my_sub_scope')['value_1'] == 1
        assert get_sub_scope_var('value_1') == 1
        assert rw.scope.get('my_sub_scope')['value_2'] == 2
        assert get_sub_scope_var('value_2') == 2
        assert rw.scope.get('my_sub_scope')['shared'] == 2
        assert get_sub_scope_var('shared') == 2

    with scope1():
        checks_inside_scope1()
        with scope2():
            checks_inside_scope2()
            with scope3():
                # scope 3 does not have a 'my_sub_scope' subscope
                # so we expect the same results as for scope 2
                checks_inside_scope2()

        checks_inside_scope1()


def test_fail():
    @rw.scope.inject
    def foo(something_to_inject):
        pass

    with pytest.raises(rw.scope.OutsideScopeError):
        foo()

    # if all arguments are provided we are ok to run outside of a scope
    foo(something_to_inject=1)


class ScopeLeakingTest(tornado.testing.AsyncTestCase):
    def test_scope_leaking(self):
        # if an exception ocurus inside a scope the scope might not
        # get clean up correctly.
        scope = rw.scope.Scope()

        with pytest.raises(NotImplementedError):
            with scope():
                raise NotImplementedError('Just some random error')

        # no we are outside of the scope
        assert rw.scope.get_current_scope() is None


class ConcurrencyTest(tornado.testing.AsyncTestCase):
    """test concurrent ioloop futures inside different scopes

    Three tests with different resolution order
    """
    def setup(self):
        """Setup two scopes and two "locks"."""
        self.scope_a = rw.scope.Scope()
        self.scope_a['name'] = 'a'
        self.lock_a = concurrent.Future()
        self.scope_b = rw.scope.Scope()
        self.scope_b['name'] = 'b'
        self.lock_b = concurrent.Future()

        @rw.scope.inject
        def get_name(name):
            return name

        @tornado.gen.coroutine
        def thread_a():
            yield self.lock_a
            raise tornado.gen.Return(get_name())

        @tornado.gen.coroutine
        def thread_b():
            yield self.lock_b
            raise tornado.gen.Return(get_name())

        with self.scope_a():
            future_a = thread_a()

        with self.scope_b():
            future_b = thread_b()

        return future_a, future_b

    @tornado.testing.gen_test
    def test_concurrent_scopes_both(self):
        """set both results before yield-ing"""
        future_a, future_b = self.setup()

        self.lock_a.set_result(None)
        self.lock_b.set_result(None)

        assert (yield future_b) == 'b'
        assert (yield future_a) == 'a'

    @tornado.testing.gen_test
    def test_concurrent_scopes_ba(self):
        """b then a"""
        future_a, future_b = self.setup()

        self.lock_b.set_result(None)
        assert (yield future_b) == 'b'

        self.lock_a.set_result(None)
        assert (yield future_a) == 'a'

    @tornado.testing.gen_test
    def test_concurrent_scopes_ab(self):
        """a then b"""
        future_a, future_b = self.setup()

        self.lock_a.set_result(None)
        assert (yield future_a) == 'a'

        self.lock_b.set_result(None)
        assert (yield future_b) == 'b'



# Copyright 2014 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import, division, print_function, with_statement
import inspect

import re

from future.builtins import range
from tornado import util
import tornado.web

from rw import scope
import rw.plugin


_rule_re = re.compile(r'''
    (?P<static>[^<]*)                           # static rule data
    <
    (?P<variable>[a-zA-Z_][a-zA-Z0-9_]*)         # variable name
    (?:
        \:                                      # variable delimiter
        (?P<converter>[a-zA-Z_][a-zA-Z0-9_]*)   # converter name
        (?:\((?P<args>.*?)\))?                  # converter arguments
    )?
    >
''', re.VERBOSE)
_simple_rule_re = re.compile(r'<([^>]+)>')


class DuplicateError(Exception):
    pass


def parse_rule(rule):
    """Parse a rule and return it as list of tuples in the form
    ``(converter, arguments, variable)``. If the converter is
    `None` it's a static url part, otherwise it's a dynamic one.

    based on werkzeug.routing
    """
    pos = 0
    end = len(rule)
    do_match = _rule_re.match
    used_names = set()
    re = []
    while pos < end:
        m = do_match(rule, pos)
        if m is None:
            break
        data = m.groupdict()
        if data['static']:
            re.append((None, None, data['static']))
        variable = data['variable']
        converter = data['converter'] or 'str'
        # if isinstance(converter, util.basestring_type):
        # TODO create hook for custom converts
        # converter = {'str': converter_default,
        #              'int': converter_int,
        #              'uint': converter_uint,
        # }[converter]
        if variable in used_names:
            raise ValueError('variable name %r used twice.' % variable)
        used_names.add(variable)
        re.append((converter, data['args'] or None, variable))
        pos = m.end()
    if pos < end:
        remaining = rule[pos:]
        if '>' in remaining or '<' in remaining:
            raise ValueError('malformed url rule: %r' % rule)
        re.append((None, None, remaining))
    return re


class NoMatchError(Exception):
    pass


class Route(object):
    def __init__(self, path):
        """Rule for `callback` matching given `path`"""
        self.path = path.rstrip('/')
        self.route = list(parse_rule(path))

    def _sort_struct(self):
        variables = [route[0] for route in self.route if route[0] is not None]
        strings = [route[2] for route in self.route if route[0] is None]
        return variables, strings

    def __lt__(self, o):
        """less than `o`

        :param Route o: other rule to compare with
        """
        if self == o:
            return False

        variables, strings = self._sort_struct()
        variables_o, strings_o = o._sort_struct()
        if len(variables) != len(variables_o):
            return len(variables) < len(variables_o)

        if len(''.join(strings)) != len(''.join(strings_o)):
            return len(''.join(strings)) > len(''.join(strings_o))

        for i in range(len(strings)):
            if strings[i] != strings_o[i]:
                return strings[i] < strings_o[i]

        # strings are the same, so check variables for non-default parser
        for i in range(len(variables)):
            if variables[i] != 'str' and variables_o[i] == 'str':
                return True
        return False

    def __gt__(self, o):
        """greater than `o`

        :param Route o: other rule to compare with
        """
        if self == o:
            return False
        return o < self

    def __eq__(self, o):
        """equal to `o`

        :param Route o: other rule to compare with
        """
        return self.route == o.route

    @rw.scope.inject
    def match(self, test_path, scope):
        arguments = {}
        for converter_name, args, data in self.route:
            if converter_name:
                converters = rw.scope.get('rw.routing:converters')
                converter = converters.get(converter_name)
                if converter is None:
                    raise AttributeError('No converter for {} available'.format(converter_name))
                try:
                    consumed, arguments[data] = converter(test_path)
                except NoMatchError:
                    return None
            elif not test_path.startswith(data):
                return None
            else:
                consumed = len(data)
            test_path = test_path[consumed:]
        if not test_path:
            return arguments
        return None

    def get_path(self, values=None):
        if values is None:
            values = {}
        re = []
        for converter, args, data in self.route:
            if converter:
                re.append(util.unicode_type(values[data]))
            else:
                re.append(data)
        return ''.join(re)

    def __repr__(self):
        return '<Rule "%s">' % self.path


def _generate_request_handler_proxy(handler_class, handler_args, name):
    """When a tornado.web.RequestHandler gets mounted we create a launcher function"""

    @scope.inject
    def request_handler_wrapper(app, handler, **kwargs):
        handler = handler_class(app, handler.request, **handler_args)
        handler._execute([], **kwargs)
    request_handler_wrapper.__name__ = name
    request_handler_wrapper.handler_class = handler_class
    request_handler_wrapper.handler_args = handler_args

    return request_handler_wrapper


class RoutingTable(dict):
    def __init__(self, name):
        dict.__init__(self)
        self.name = name
        self.prefix = ''
        self.sub_rt = []  # child routing tables
        self.fn_namespace = {}
        for method in ['get', 'post', 'put', 'delete', 'options']:
            self[method] = []

    def setup(self):
        """setup routing table"""
        # get all routes from submodules
        for prefix, routes in self.sub_rt:
            routes.prefix = self.prefix + prefix
            routes.setup()

            fn_name_prefixes = {}
            for fn_key, fn in routes.fn_namespace.items():
                self.fn_namespace[routes.name + '.' + fn_key] = fn
                fn_prefix = routes.name
                if '.' in fn_key:
                    fn_prefix += '.' + fn_key.rsplit('.', 1)[0]
                fn_name_prefixes[fn] = fn_prefix

            for key in self:
                funcs = set(rule[1] for rule in self[key])
                for route, route_module, fn in routes.get(key, []):
                    if fn not in funcs:
                        new_route = Route(prefix + route.path)
                        fn.rw_route = new_route
                        fn_name_prefix = fn_name_prefixes[fn]
                        self[key].append((new_route, fn_name_prefix, fn))

        # sort all rules
        for key in self:
            self[key].sort(key=lambda rule: rule[0])

    def add_route(self, method, path, module, fn):
        route = Route(path)
        self.setdefault(method, []).append((route, '', fn))
        # if fn.__name__ in self.fn_namespace:
        #     msg = 'Module already contains route with name {}'.format(fn.__name__)
        #     raise DuplicateError(msg)
        fn.rw_route = route
        self.fn_namespace[fn.__name__] = fn
        return route

    def get_path(self, func, kwargs):
        if func not in self.fn_namespace:
            # TODO do something more sensitve
            # - Log warning
            # - return actual 404 url
            return '404'
        return self.fn_namespace[func].rw_route.get_path(kwargs)

    def add_request_handler(self, path, module, handler_args, name):
        if name is None:
            name = module.__name__
        for method in ['get', 'post', 'put', 'delete']:
            proxy = _generate_request_handler_proxy(module,
                                                    handler_args,
                                                    name + '_' + method)
            if hasattr(module, method):
                route = self.add_route(method, path, module, proxy)
                unbount_method = getattr(module, method)
                if hasattr(unbount_method, '__func__'):
                    # python 2
                    # it is not possible to write to the unbount_method
                    # but writing to the underlying im_func works
                    unbount_method.__func__.rw_route = route
                else:
                    # python 3
                    unbount_method.rw_route = route

    def add_child(self, path, rt):
            # module.routes.prefix = path
        self.sub_rt.append((path, rt))

    def find_route(self, method, path):
        for rule, name_prefix, fn in self[method.lower()]:
            args = rule.match(path)
            if args is not None:
                return name_prefix, fn, args
        return None, None, None


def converter_default(data):
    length = data.find('/')
    if length == 1 or len(data) == 0:
        raise NoMatchError()
    elif length < 0:
        length = len(data)
    return length, util.unicode_type(data[:length])


def converter_path(data):
    """consume rest of the path"""
    return len(data), util.unicode_type(data)


NON_INT = re.compile('[^0-9-]')


def converter_int(data):
    length = NON_INT.search(data)
    length = length.start() if length else len(data)
    if length <= 0:
        raise NoMatchError()
    return length, int(data[:length])


NON_UINT = re.compile('[^0-9]')


def converter_uint(data):
    length = NON_UINT.search(data)
    length = length.start() if length else len(data)
    if length <= 0:
        raise NoMatchError()
    return length, int(data[:length])


plugin = rw.plugin.Plugin(__name__)


@plugin.init
def init(scope):
    scope.setdefault('rw.routing:converters', {}).update({
        'str': converter_default,
        'int': converter_int,
        'uint': converter_uint,
        'path': converter_path
    })



import tornado.testing

import rw.plugin
import rw.scope


class MyTestCase(tornado.testing.AsyncTestCase):
    def test_basic(self):
        scope = rw.scope.Scope()

        plugin = rw.plugin.Plugin('rw.test')

        @plugin.init
        def init(scope):
            scope['foo'] = 1

        with scope():
            scope.activate(plugin, callback=self.inside_scope)
        self.wait()

    @rw.scope.inject
    def inside_scope(self, result, scope):
        assert scope.get('foo') == 1
        self.stop()



# Copyright 2014 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# PYTHON_ARGCOMPLETE_OK

"""rueckenwind command line tool"""
from __future__ import absolute_import, division, print_function, with_statement

import shutil
import sys
import os
import argparse
import logging

import argcomplete
import pkg_resources
import tornado.httpserver
import tornado.ioloop
import tornado.autoreload
import tornado.util
import jinja2

import rw
import rw.scope
import rw.server
import rw.httpbase


ARG_PARSER = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawTextHelpFormatter)
SUB_PARSER = ARG_PARSER.add_subparsers(help='Command help')


def command(func):
    """Decorator for CLI exposed functions"""
    func.parser = SUB_PARSER.add_parser(func.__name__, help=func.__doc__)
    func.parser.set_defaults(func=func)
    return func


def create_skel(src, dst, data):
    """generate skeleton rw project"""
    dst = dst.format(**data)
    if not os.path.exists(dst):
        os.makedirs(dst)

    for fname in pkg_resources.resource_listdir(__name__, src):
        path = os.path.join(src, fname)
        dest_fname = os.path.join(dst, fname.format(**data))
        ext = os.path.splitext(fname)[1].lower()

        if pkg_resources.resource_isdir(__name__, path):
            create_skel(path, dst + '/' + fname, data)
        elif ext in ('.py', '.svg', '.yml'):
            tpl = pkg_resources.resource_string(__name__, path)
            tpl = tpl.decode('utf-8')
            content = jinja2.Template(tpl).render(**data)
            open(dest_fname, 'w').write(content.encode('utf-8'))
        elif ext in ('.css', '.png', '.html'):
            src = pkg_resources.resource_filename(__name__, path)
            shutil.copy(src, dest_fname)


@command
def skel(args):
    """Generate a skeleton project"""
    if args.name:
        name = args.name
    else:
        name = raw_input('Name: ')
    if os.path.exists(name):
        print('Already exists')
        sys.exit(1)

    os.mkdir(name)
    create_skel('skel', os.path.abspath(name), {'name': name})

skel.parser.add_argument('--name', type=str,
                         help='Name of the module to be created')


@command
def serv(args):
    """Serve a rueckenwind application"""
    if not args.no_debug:
        tornado.autoreload.start()

    extra = []

    if sys.stdout.isatty():
        # set terminal title
        sys.stdout.write('\x1b]2;rw: {}\x07'.format(' '.join(sys.argv[2:])))

    if args.cfg:
        extra.append(os.path.abspath(args.cfg))

    listen = (int(args.port), args.address)
    ioloop = tornado.ioloop.IOLoop.instance()
    setup_app(app=args.MODULE, extra_cfg=extra, ioloop=ioloop, listen=listen)
    ioloop.start()


def setup_app(app, extra_cfg=None, ioloop=None, listen=None):
    if ioloop is None:
        ioloop = tornado.ioloop.IOLoop.instance()
    if extra_cfg is None:
        extra_cfg = []

    if isinstance(app, tornado.util.basestring_type):
        module_path = app
        module_name = 'root'
        if ':' in module_path:
            module_path, module_name = module_path.split(':', 1)
        module_path = module_path.replace('/', '.').strip('.')
        module = __import__(module_path, fromlist=[module_name])
        module = getattr(module, module_name)
        app = rw.httpbase.Application(root=module)

    server = tornado.httpserver.HTTPServer(app)
    if listen:
        server.listen(*listen)

    scope = rw.scope.Scope()
    with scope():
        ioloop.run_sync(rw.server.start)
    return app.scope


serv.parser.add_argument('-p', '--port', type=str, default='8000',
                         help='Specify port to run http server on')
serv.parser.add_argument('-a', '--address', type=str,
                         help='Specify port to run http server on')
serv.parser.add_argument('--no-debug', action='store_true',
                         help='Run in production mode')
serv.parser.add_argument('-c', '--cfg', type=str,
                         help='Additional config to load')
serv.parser.add_argument('MODULE',
                         help='Module to serve')


def main():
    """Entry point of rw cli"""
    # check logging
    log_level = os.environ.get('LOG_LEVEL', 'INFO')
    logging.basicConfig(level=getattr(logging, log_level),
                        format='%(asctime)s %(name)s[%(levelname)s] %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')

    current_path = os.path.abspath('.')
    if current_path not in sys.path:
        sys.path.insert(0, current_path)
    argcomplete.autocomplete(ARG_PARSER)
    args = ARG_PARSER.parse_args()
    args.func(args)



import imp

import pkg_resources
import rw.testing

from . import example


class HTTPServerTest(rw.testing.AsyncHTTPTestCase):
    def get_app(self):
        return rw.httpbase.Application(root=imp.reload(example).root)

    def test_basic_routing(self):
        response = self.fetch('/')
        assert response.code == 200
        assert response.body.decode('utf-8') == u'Hello rw.http'

        response = self.fetch('/lazy')
        assert response.code == 200
        assert response.body.decode('utf-8') == u'Hello lazy rw.http'

        response = self.fetch('/otherplace')
        assert response.code == 200
        assert response.body.decode('utf-8') == u'other'

        response = self.fetch('/user/me')
        assert response.code == 200
        assert response.body.decode('utf-8') == u'Hello me'

        response = self.fetch('/user/you')
        assert response.code == 200
        assert response.body.decode('utf-8') == u'Hello you'

        response = self.fetch('/nowhere')
        assert response.code == 404

        response = self.fetch('/put')
        assert response.code == 404

    def test_template(self):
        response = self.fetch('/foo')
        assert response.code == 200
        var, hello_world, hello_world2 = response.body.decode('utf-8').strip().split('\n')
        assert var == u'static_value: 42 == 42'

        hello_world_content = pkg_resources.resource_string(
            'test.example', 'static/test.example/hello_world.txt')
        hello_world_response = self.fetch(hello_world)
        assert hello_world_response.code == 200
        assert hello_world_response.body == hello_world_content

        hello_world_response2 = self.fetch(hello_world2)
        assert hello_world_response2.code == 200
        assert hello_world_response2.body == hello_world_content

    def test_different_methods(self):
        response = self.fetch('/', method='POST', body='')
        assert response.body.decode('utf-8') == u'root POST'

        response = self.fetch('/put', method='PUT', body='')
        assert response.body.decode('utf-8') == u'put'

        response = self.fetch('/delete', method='DELETE')
        assert response.body.decode('utf-8') == u'delete'

        response = self.fetch('/options', method='OPTIONS')
        assert response.body.decode('utf-8') == u'options'

    def test_mount_tornado_handler(self):
        response = self.fetch('/tornado', method='GET')
        assert response.body.decode('utf-8') == u'Tornado GET'

    def test_static(self):
        # from static folder
        response = self.fetch('/static/hash/test.example/hello_world.txt', method='GET')
        assert response.body.decode('utf-8') == u'Hello Static World'

        # from static2 folder
        response = self.fetch('/static/hash/test.example/hallo_welt.txt', method='GET')
        assert response.body.decode('utf-8') == u'Guten Tag.'

        # from static2 folder (higher in config file)
        response = self.fetch('/static/hash/test.example/overwrite.txt', method='GET')
        assert response.body.decode('utf-8') == u'Overwrite'

    def test_url_for_inside_submodule(self):
        response = self.fetch('/sub', method="GET")
        assert response.body.decode('utf-8') == '/sub\n/sub'


def generate_route_func(name):
    def f(x):
        return x

    f.__name__ = name
    return f


def generate_handler_func(route_func, path, name=None):
    if name is None:
        name = route_func.__name__ + '_' + {'/': 'index'}[path]
    f = generate_route_func(name)
    return route_func(path)(f)



import pytest
import tornado.testing

import rw.routing
import rw.scope

from .common import generate_route_func


def generate_rule(route):
    return rw.routing.Route(route)


def test_parse_rule():
    assert rw.routing.parse_rule('/asdf/') == [(None, None, '/asdf/')]
    rule = rw.routing.parse_rule('/foo/<name>/bar')
    assert rule[0] == (None, None, '/foo/')
    assert rule[1] == ('str', None, 'name')
    assert rule[2] == (None, None, '/bar')


def test_parse_rule_errors():
    with pytest.raises(ValueError):
        # the argument name "name" is used twice
        list(rw.routing.parse_rule('/<name>/<name>/asd'))

    with pytest.raises(ValueError):
        # missing > after name
        list(rw.routing.parse_rule('/<name/asd'))


def test_rule_compare():
    for rule_0, rule_1 in (
        (generate_rule('/name'), generate_rule('/<something>')),
        (generate_rule('/name'), generate_rule('/na<something>')),
        (generate_rule('/name<something>'), generate_rule('/na<something>')),
        (generate_rule('/<something:int>'), generate_rule('/<something>')),
    ):
        assert rule_0 < rule_1
        assert rule_1 > rule_0

    assert not generate_rule('/name') < generate_rule('/name')
    assert not generate_rule('/name') > generate_rule('/name')


def test_rule_eq():
    rules = ['/', '/foo', '/name/<name>/photo', '/name/<name>/photo/<num:int>']
    # rules must implement __eq__
    for rule in rules:
        assert generate_rule(rule) == generate_rule(rule)

    assert generate_rule('/') != generate_rule('/foo')
    assert (generate_rule('/name/<name>/photo/<num>')
            != generate_rule('/name/<name>/photo/<num:int>'))


def test_rule_sorting():
    rules = [
        generate_rule('/name'),
        generate_rule('/'),
        generate_rule('/name/<name>/photo'),
        generate_rule('/name/<else>'),
        generate_rule('/<something>'),
    ]
    rules2 = rules[:]
    rules2.sort()
    assert rules == rules2

    rules3 = rules[:]
    rules3.reverse()
    rules3.sort()
    assert rules == rules3


def test_reverse_path():
    assert generate_rule('/').get_path() == '/'
    assert generate_rule('/somewhere').get_path() == '/somewhere'
    assert generate_rule('/user/<user>').get_path({'user': 'dino'}) == '/user/dino'


def test_converter_default():
    assert (3, 'foo') == rw.routing.converter_default('foo')
    assert (3, 'foo') == rw.routing.converter_default('foo/bar')


def test_convert_int():
    assert rw.routing.converter_int('123') == (3, 123)
    assert rw.routing.converter_int('4321Hello World') == (4, 4321)
    assert rw.routing.converter_int('-1431') == (5, -1431)
    with pytest.raises(rw.routing.NoMatchError):
        rw.routing.converter_int('foo')


def test_convert_uint():
    assert rw.routing.converter_uint('123') == (3, 123)
    assert rw.routing.converter_uint('4321Hello World') == (4, 4321)

    with pytest.raises(rw.routing.NoMatchError):
        assert rw.routing.converter_uint('-1431') == (5, -1431)


def test_submodules():
    rt0 = rw.routing.RoutingTable('root')
    rt0.add_route('get', '/', 0, generate_route_func('index'))
    rt1 = rw.routing.RoutingTable('sub')
    rt1.add_route('get', '/', 1, generate_route_func('index'))
    rt2 = rw.routing.RoutingTable('subsub')
    rt2.add_route('get', '/', 2, generate_route_func('index'))
    rt2.add_route('get', '/fun', 2, generate_route_func('fun'))

    rt0.add_child('/sub', rt1)
    rt1.add_child('/subsub', rt2)
    rt0.setup()

    scope = rw.scope.Scope()
    with scope():
        prefix, func, args = rt0.find_route('get', '/')
        assert prefix == ''
        assert func.__name__ == 'index'

        prefix, func, args = rt0.find_route('get', '/sub')
        assert prefix == 'sub'
        assert func.__name__ == 'index'

        prefix, func, args = rt0.find_route('get', '/sub/subsub')
        assert prefix == 'sub.subsub'
        assert func.__name__ == 'index'

        prefix, func, args = rt0.find_route('get', '/sub/subsub/fun')
        assert prefix == 'sub.subsub'
        assert func.__name__ == 'fun'


class MyTestCase(tornado.testing.AsyncTestCase):
    def test_rule_match(self):
        # match must be used inside scope with rw.routing:plugin activated
        scope = rw.scope.Scope()

        with scope():
            scope.activate(rw.routing.plugin, callback=self.inside_scope)
        self.wait()

    def inside_scope(self, result):
        assert generate_rule('/').match('/') == {}
        assert generate_rule('/').match('/asd') is None
        assert generate_rule('/<foo>').match('/asd') == {'foo': 'asd'}
        assert generate_rule('/<foo>').match('/foo/bar') is None
        assert generate_rule('/<foo:path>').match('/foo/bar') == {'foo': 'foo/bar'}
        self.stop()



import tornado.gen
from tornado.gen import *


def coroutine(func, replace_callback=True):
    """Decorator for asynchronous generators.

    Any generator that yields objects from this module must be wrapped
    in either this decorator or `engine`.

    Coroutines may "return" by raising the special exception
    `Return(value) <Return>`.  In Python 3.3+, it is also possible for
    the function to simply use the ``return value`` statement (prior to
    Python 3.3 generators were not allowed to also return values).
    In all versions of Python a coroutine that simply wishes to exit
    early may use the ``return`` statement without a value.

    Functions with this decorator return a `.Future`.  Additionally,
    they may be called with a ``callback`` keyword argument, which
    will be invoked with the future's result when it resolves.  If the
    coroutine fails, the callback will not be run and an exception
    will be raised into the surrounding `.StackContext`.  The
    ``callback`` argument is not visible inside the decorated
    function; it is handled by the decorator itself.

    From the caller's perspective, ``@gen.coroutine`` is similar to
    the combination of ``@return_future`` and ``@gen.engine``.
    """
    re = tornado.gen._make_coroutine_wrapper(func, replace_callback=True)
    re._rw_wrapped_function = func
    return re


from tornado.testing import AsyncHTTPTestCase, AsyncTestCase, ExpectLog, gen_test

import rw.httpbase


class HelloWorldHandler(rw.httpbase.RequestHandler):
    def handle_request(self):
        self.finish('Hello World')


class HTTPServerTest(AsyncHTTPTestCase):
    def get_app(self):
        return rw.httpbase.Application(handler=HelloWorldHandler)

    def test_hello_world(self):
        response = self.fetch('/')
        assert response.body.decode('utf-8') == u'Hello World'



# Copyright 2014 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""rw event system.


Signal

"""
from __future__ import absolute_import, division, print_function, with_statement

import traceback

from tornado import gen
import rw.scope


class EventException(Exception):
    def __init__(self, exceptions):
        self.exceptions = exceptions
        message = '{} exceptions encountered:\n'.format(len(exceptions))
        for func, e in exceptions:
            message += '{}:\n{}'.format(func, e)
        Exception.__init__(self, ''.join(message))


class Event(set):
    def __init__(self, name, accumulator=None):
        super(Event, self).__init__()
        self.name = name
        self.accumulator = accumulator

    @gen.coroutine
    def __call__(self, *args, **kwargs):
        scope = rw.scope.get_current_scope()
        rw_tracing = scope.get('rw_trace', None) if scope else None

        re = []
        exceptions = []
        futures = []
        for func in self:
            try:
                result = func(*args, **kwargs)
                if isinstance(result, gen.Future):
                    # we are not waiting for future objects result here
                    # so they evaluate in parallel
                    futures.append((func, result))
                else:
                    re.append(result)
            except Exception:
                exceptions.append((func, traceback.format_exc()))

        # wait for results
        for func, future in futures:
            try:
                if not future.done():
                    yield future
                re.append(future.result())

            except Exception:
                exceptions.append((func, traceback.format_exc()))

        if exceptions:
            raise EventException(exceptions)

        # apply accumolator
        if self.accumulator:
            re = self.accumulator(re)

        raise gen.Return(re)

    def add(self, func):
        assert callable(func)
        set.add(self, func)
        return func



from __future__ import absolute_import, division, print_function, with_statement

import pytest
import rw.event

import tornado.gen
import tornado.testing


@tornado.gen.coroutine
def something_lazy(x):
    for i in range(x):
        yield something_lazy(0)
    raise tornado.gen.Return(x)


class MyTestCase(tornado.testing.AsyncTestCase):
    @tornado.testing.gen_test
    def test_exception_handling(self):
        MY_EVENT = rw.event.Event('MY_EVENT')

        @MY_EVENT.add
        def fail():
            1 / 0

        with pytest.raises(rw.event.EventException):
            yield MY_EVENT()

        try:
            yield MY_EVENT()
            assert False  # this line should never be reached
        except rw.event.EventException as e:
            # the original traceback should be get printed
            assert 'in fail' in str(e)  # function name of the actual exception
            assert '1 / 0' in str(e)  # the source line of the exception
            assert 'ZeroDivisionError' in str(e)

    @tornado.testing.gen_test
    def test_event_listener(self):
        MY_EVENT = rw.event.Event('MY_EVENT')
        args = []

        def listener(greeting):
            args.append(greeting)

        MY_EVENT.add(listener)

        yield MY_EVENT('Hello World')
        assert args == ['Hello World']
        MY_EVENT.remove(listener)
        assert len(MY_EVENT) == 0

    @tornado.testing.gen_test
    def test_decorator(self):
        MY_EVENT = rw.event.Event('MY_EVENT')
        args = []

        @MY_EVENT.add
        def listener(greeting):
            args.append(greeting)

        yield MY_EVENT('Hallo Welt')
        assert args == ['Hallo Welt']

    @tornado.testing.gen_test
    def test_accumulator(self):
        MY_EVENT = rw.event.Event('MY_EVENT', sum)
        MY_EVENT.add(lambda: 1)
        MY_EVENT.add(lambda: 2)

        assert (yield MY_EVENT()) == 3

    @tornado.testing.gen_test
    def test_futures(self):
        MY_EVENT = rw.event.Event('MY_EVENT')
        MY_EVENT.add(something_lazy)
        result = yield MY_EVENT(12)
        assert result == [12]

    @tornado.testing.gen_test
    def test_futures_fail(self):
        @tornado.gen.coroutine
        def something_lazy_failing():
            yield something_lazy(1)
            1 / 0

        MY_EVENT = rw.event.Event('MY_EVENT')
        MY_EVENT.add(something_lazy_failing)

        with pytest.raises(rw.event.EventException):
            yield MY_EVENT()

        try:
            yield MY_EVENT()
            assert False  # this line should never be reached
        except rw.event.EventException as e:
            # the original traceback should be get printed
            assert 'in something_lazy_failing' in str(e)  # function name of the actual exception
            assert '1 / 0' in str(e)  # the source line of the exception
            assert 'ZeroDivisionError' in str(e)


from __future__ import absolute_import, division, print_function, with_statement

import os
import hashlib
import base64

import pkg_resources
import tornado.web
import types
from tornado.util import bytes_type

import rw.plugin
import rw.scope


class StaticHandler(tornado.web.StaticFileHandler):
    def get(self, path, include_body=True, h=None):
        # TODO only in development mode
        self.set_header('Cache-Control', 'no-cache, no-store, must-revalidate')
        self.set_header('Pragma', 'no-cache')
        self.set_header('Expires', '0')
        return super(StaticHandler, self).get(path, include_body)

    @classmethod
    def get_absolute_path(cls, roots, path):
        """Returns the absolute location of ``path`` relative to one of
        the ``roots``.

        ``roots`` is the path configured for this `StaticFileHandler`
        (in most cases the ``static_path`` `Application` setting).
        """
        for root in roots:
            abspath = os.path.abspath(os.path.join(root, path))
            if abspath.startswith(root) and os.path.exists(abspath):
                return abspath
        # XXX TODO
        return 'file-not-found'

    def validate_absolute_path(self, roots, absolute_path):
        """Validate and return the absolute path.

        ``root`` is the configured path for the `StaticFileHandler`,
        and ``path`` is the result of `get_absolute_path`

        This is an instance method called during request processing,
        so it may raise `HTTPError` or use methods like
        `RequestHandler.redirect` (return None after redirecting to
        halt further processing).  This is where 404 errors for missing files
        are generated.
        """
        if absolute_path is None:
            raise tornado.web.HTTPError(404)

        for root in roots:

            if (absolute_path + os.path.sep).startswith(root):
                break
        else:
            raise tornado.web.HTTPError(403, "%s is not in root static directory",
                                        self.path)

        if not os.path.isfile(absolute_path):
            raise tornado.web.HTTPError(403, "%s is not a file", self.path)

        return absolute_path


def file_hash(content):
    """
    :param str|FileIO content: The content to hash, either as string or as file-like object
    """
    h = hashlib.sha256()
    if isinstance(content, bytes_type):
        h.update(content)
    else:
        data = True
        while data:
            data = content.read(1024 * 1024)
            h.update(data)
    h_digest = h.digest()
    # base64url
    # | char | substitute |
    # |   +  |    -       |
    # |   /  |    _       |
    #

    return base64.b64encode(h_digest, altchars=b'-_').rstrip(b'=')


class Static(object):
    def __init__(self):
        self.handlers = []

    def __call__(self, path):
        """returns url for static path"""
        if not path.startswith('/'):
            uri = '/static/' + path
        else:
            uri = path

        for base_uri, handler_class, roots in self.handlers:
            if uri.startswith('/' + base_uri + '/'):
                path = uri[len(base_uri) + 2:]  # remove /base_uri/
                abs_path = handler_class.get_absolute_path(roots, path)
                if not abs_path:
                    raise Exception('File Not Found %s' % repr(path))
                content = handler_class.get_content(abs_path)
                if isinstance(content, types.GeneratorType):
                    try:
                        content = b''.join(content)
                    except IOError:
                        raise Exception('File Not Found %s' % repr(path))
                break
        else:
            # XXX todo: something more sensitive
            raise Exception('File Not Found %s' % repr(path))

        h = file_hash(content)[:6]
        return '/{}/{}/{}'.format(base_uri, h, path)

    def setup(self):
        self.handlers.sort(key=lambda x: len(x[0]), reverse=True)

plugin = rw.plugin.Plugin(__name__)


@plugin.init
def init(scope, app, settings):
    """serve static files"""
    cfg = settings.get('rw.static', {})
    static = Static()
    scope['static'] = static
    scope['template_env'].globals['static'] = static
    for base_uri, sources in cfg.items():
        full_paths = []
        for source in sources:
            if isinstance(source, dict):
                full_path = source['path']
                full_paths.append(full_path.format(**os.environ))
                continue
            elif ',' in source:
                module_name, path = [part.strip() for part in source.split(',')]
            else:
                module_name = source
                path = 'static'
            full_path = pkg_resources.resource_filename(module_name, path)
            full_paths.append(full_path)

        app.root.mount('/' + base_uri + '/<h>/<path:path>',
                       StaticHandler, {'path': full_paths},
                       name='static_' + base_uri.replace('.', '_'))

        static.handlers.append((base_uri, StaticHandler, full_paths))
    static.setup()



from collections import namedtuple

import pytest
import tornado.testing

import rw.scope
import rw.http
import rw.routing

from .common import generate_handler_func

## TODO
# def test_duplicated_functions():
#     m = rw.http.Module('test')
#
#     @m.get('/')
#     def foo():
#         pass
#
#     with pytest.raises(rw.routing.DuplicateError):
#         @m.get('/something_else')
#         def foo():
#             pass


class HTTPTest(tornado.testing.AsyncTestCase):
    def test_http(self):
        self.scope = rw.scope.Scope()
        with self.scope():
            self.scope.activate(rw.routing.plugin, callback=self.http)
        self.wait()

    def http(self, _):
        sub = rw.http.Module('sub')
        sub_index = generate_handler_func(sub.get, '/')
        sub_fun = generate_handler_func(sub.get, '/fun', 'fun')
        sub_posts = generate_handler_func(sub.get, '/<user_name:str>/posts', 'sub_posts')

        subsub = rw.http.Module('subsub')
        subsub_index = generate_handler_func(subsub.get, '/')
        subsub_fun = generate_handler_func(subsub.get, '/fun', 'fun')
        sub.mount('/sub', subsub)

        sub2 = rw.http.Module('sub2')
        sub2_index = generate_handler_func(sub2.get, '/')

        m = rw.http.Module('test')
        index = generate_handler_func(m.get, '/')
        index_post = generate_handler_func(m.post, '/')
        index_put = generate_handler_func(m.put, '/')
        index_delete = generate_handler_func(m.delete, '/')
        user = generate_handler_func(m.get, '/user/<user_name:str>', 'user_page')
        m.mount('/sub', sub)
        m.mount('/sub2', sub2)
        routes = m.setup()

        # mock app object
        self.scope['app'] = namedtuple('Application', 'root')(m)

        ## test find_route
        assert routes.find_route('get', '/')[1] == index
        assert routes.find_route('post', '/')[1] == index_post
        assert routes.find_route('put', '/')[1] == index_put
        assert routes.find_route('delete', '/')[1] == index_delete

        assert routes.find_route('get', '/user/joe') == ('', user, {'user_name': 'joe'})

        ## test find_route for mounts
        assert routes.find_route('get', '/sub')[1] == sub_index
        assert routes.find_route('get', '/sub2')[1] == sub2_index

        ## test url_for
        assert rw.http.url_for(index) == '/'
        assert rw.http.url_for(index_post) == '/'

        assert rw.http.url_for(user, user_name='joe') == '/user/joe'

        ## test url_for within mount
        assert rw.http.url_for(sub_index) == '/sub'
        assert rw.http.url_for(sub_fun) == '/sub/fun'
        assert rw.http.url_for(sub_posts, user_name='bob') == '/sub/bob/posts'

        ## test url_for with strings
        assert rw.http.url_for('get_index') == '/'
        assert rw.http.url_for('post_index') == '/'
        assert rw.http.url_for('user_page', user_name='joe') == '/user/joe'

        ## test url_for with strings and mount prefix
        assert rw.http.url_for('sub.get_index') == '/sub'
        assert rw.http.url_for('sub.fun') == '/sub/fun'
        assert rw.http.url_for(sub_posts, user_name='bob') == '/sub/bob/posts'
        assert rw.http.url_for('sub.subsub.get_index') == '/sub/sub'

        ## test url_for with relative string
        self.scope['rw.routing.prefix'] = ''  # mock request
        assert rw.http.url_for('.get_index') == '/'
        self.scope['rw.routing.prefix'] = 'sub'  # mock request
        assert rw.http.url_for('.get_index') == '/sub'
        assert rw.http.url_for('.fun') == '/sub/fun'
        assert rw.http.url_for('.subsub.get_index') == '/sub/sub'
        assert rw.http.url_for('.subsub.fun') == '/sub/sub/fun'

        self.scope['rw.routing.prefix'] = 'sub.subsub'  # mock request
        assert rw.http.url_for('.fun') == '/sub/sub/fun'

        self.stop()

    def test_mount(self):
        scope = rw.scope.Scope()
        with scope():
            scope.activate(rw.routing.plugin, callback=self.mount)
        self.wait()

    def mount(self, _):
        m = rw.http.Module('test')
        index = generate_handler_func(m.get, '/')

        sub = rw.http.Module('test')
        sub_index = generate_handler_func(sub.get, '/')
        m.mount('/foo', sub)

        routes = m.setup()

        assert routes.find_route('get', '/')[1] == index
        assert routes.find_route('get', '/foo')[1] == sub_index
        self.stop()

    def test_mount_variables(self):
        scope = rw.scope.Scope()
        with scope():
            scope.activate(rw.routing.plugin, callback=self.mount)
        self.wait()

    def mount_variables(self, _):
        m = rw.http.Module('test')
        index = generate_handler_func(m.get, '/')

        sub = rw.http.Module('test')
        sub_index = generate_handler_func(sub.get, '/')
        m.mount('/foo/<var>', sub)

        routes = m.setup()

        assert routes.find_route('get', '/')[1] == index
        assert routes.find_route('get', '/foo/bar')[1] == sub_index
        self.stop()


import tornado.testing

import rw.server


class AsyncHTTPTestCase(tornado.testing.AsyncHTTPTestCase):
    def get_http_server(self):
        # setup ._app before creating http server
        rw.server.start(callback=self.stop)
        self.wait()
        return super(AsyncHTTPTestCase, self).get_http_server()


"""Rueckenwind tempating

Rueckenwind does not contain it's own templating engine
but is based on Jinja2.

Added methods inside functions:
 * url_for
 * handler
"""
import json

import tornado.util
import jinja2

import rw.http


def create_template_env(pkgs):
    loaders = [jinja2.PackageLoader(pkg, 'templates') for pkg in pkgs]
    template_env = jinja2.Environment(
        loader=jinja2.ChoiceLoader(loaders),
        extensions=['jinja2.ext.loopcontrols',
                    'jinja2.ext.i18n'],
    )
    template_env.globals['url_for'] = rw.http.url_for
    template_env.filters['json'] = json.dumps
    template_env.globals['str'] = tornado.util.unicode_type

    # some more default functions
    # template_env.globals['enumerate'] = enumerate
    # template_env.globals['isinstance'] = isinstance
    # template_env.globals['len'] = len
    # # default types
    # template_env.globals['int'] = int

    # template_env.globals['unicode'] = unicode
    # template_env.globals['list'] = list
    # template_env.globals['tuple'] = tuple
    # template_env.globals['dict'] = dict
    # template_env.globals['set'] = set
    # template_env.globals['basestring'] = basestring
    # template_env.globals['urlencode'] = urlencode
    # filter

    return template_env


"""An example using as many different features of rueckenwind as possible

"""
import os
import time

import tornado.web
import tornado.ioloop
import rw.testing
import rw.http
import rw.httpbase
from rw import gen


BASE_PATH = os.path.abspath(os.path.dirname(__file__))


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.write("Tornado GET")

    def post(self):
        self.write("Tornado POST")


sub = rw.http.Module(name='submodule', resources='test.example')


@sub.get('/')
def sub_index():
    sub.render_template('sub.html')


root = rw.http.Module('test.example')
root.mount('/tornado', MainHandler)
root.mount('/sub', sub)


@root.init
def init(template_env):
    template_env.globals['static_value'] = 42


@root.get('/')
def index(handler):
    handler.finish('Hello rw.http')


@root.get('/lazy')
@gen.coroutine
def lazy(handler):
    yield gen.Task(tornado.ioloop.IOLoop.current().add_timeout, time.time())
    handler.finish('Hello lazy rw.http')


@root.post('/')
def root_submit(handler):
    handler.finish('root POST')


@root.get('/user/<name>')
def user_page(handler, name):
    handler.finish('Hello ' + name)


@root.get('/otherplace')
def other(handler):
    handler.finish('other')


@root.put('/put')
def put(handler):
    handler.finish('put')


@root.delete('/delete')
def delete(handler):
    handler.finish('delete')


@root.options('/options')
def options(handler):
    handler.finish('options')


@root.get('/foo')
def some_page():
    root.render_template('index.html')



"""An example using as many different features of rueckenwind as possible

"""
import rw.http
from rw import gen


root = rw.http.Module('{{name}}')


@root.get('/')
def index(handler):
    root.render_template('index.html')



# Copyright 2015 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import, division, print_function, with_statement

import os
import inspect

import tornado.web
import tornado.httpserver
import tornado.ioloop
from tornado import gen
from tornado import iostream
from tornado.web import HTTPError
from tornado.concurrent import is_future
from tornado.web import _has_stream_request_body

import rw.cfg
import rw.scope
import rw.routing
import rw.template
import rw.server
import rw.event


PRE_REQUEST = rw.event.Event('httpbase.pre_request')
POST_REQUEST = rw.event.Event('httpbase.post_request')


class Application(object):
    def __init__(self, handler=None, root=None):
        """rueckenwind Application to plug into tornado's httpserver.

        Either `root` or `handler` must be specified.

        :param rw.http.Module root: The root module to serve
        :param handler: The request handler (should subclass `tornado.web.RequestHandler`)
        """
        self.io_loop = tornado.ioloop.IOLoop.current()
        self.settings = {}
        self.rw_settings = {}
        self.root = root
        self.scope = rw.scope.Scope()
        self.scope['app'] = self
        if self.root:
            self.handler = handler if handler is not None else RequestHandler
            pkgs = [root.name]  # TODO, Breadth-first search for dependencies
            self.scope['template_env'] = rw.template.create_template_env(pkgs)
            self.scope['template_env'].globals['app'] = self
        else:
            self.handler = handler
            assert handler is not None

        self._wsgi = False  # wsgi is not supported
        # compatibility so we can mount tornado RequestHandlers
        self.ui_modules = {}
        self.ui_methods = {}
        rw.server.PHASE_CONFIGURATION.add(self.configure)

    def configure(self):
        with self.scope():
            return self._scoped_configure()

    @gen.coroutine
    def _scoped_configure(self):
        yield rw.scope.setup_app_scope(self.root.name, self.scope)
        self.rw_settings = self.scope['settings']
        cfg_rw_http = self.rw_settings.setdefault('rw.http', {})
        cfg_rw_http['live_settings'] = self.settings
        self._configure_cookie_secret()

        yield self.scope.activate(self.root)

    def _configure_cookie_secret(self):
        cfg = self.rw_settings['rw.http']
        if 'cookie_secret' in cfg:
            if 'file' in cfg['cookie_secret']:
                cs_path = cfg['cookie_secret']['file']
                cs_path = cs_path.format(**os.environ)
                if os.path.exists(cs_path):
                    cookie_secret = open(cs_path).read().strip()
                else:
                    cs_dir = os.path.dirname(cs_path)
                    if not os.path.exists(cs_dir):
                        os.makedirs(cs_dir)
                    cookie_secret = os.urandom(32)
                    open(cs_path, 'w').write(cookie_secret)
            elif 'random' in cfg['cookie_secret'] and cfg['cookie_secret']['random']:
                cookie_secret = os.urandom(32)
            cfg['live_settings']['cookie_secret'] = cookie_secret

    def __call__(self, request):
        """Called by `tornado.httpserver.HTTPServer` to handle a request."""
        with self.scope():
            request_scope = rw.scope.Scope()
            with request_scope():
                request_handling = self._handle_request(request_scope, request)
                self.io_loop.add_future(request_handling, self._request_finished)

    @gen.coroutine
    def _handle_request(self, request_scope, request):
        handler = self.handler(self, request)
        request_scope['handler'] = handler
        yield PRE_REQUEST()
        yield handler._execute([])
        yield POST_REQUEST()

    def _request_finished(self, request_future):
        # access result to throw exceptions that might have occurred during
        # request handling
        request_future.result()

    def log_request(self, request):
        # TODO print(request)
        pass


class RequestHandler(tornado.web.RequestHandler, dict):
    def __init__(self, application, request, **kwargs):
        # The super class is not called since it creates
        # some structures we do not care about.  Since
        # the "not caring" leads to memory leaks they
        # are not created in the first place.

        self.application = application
        self.request = request
        self._headers_written = False
        self._finished = False
        self._auto_finish = False  # vanilla tornado defaults to True
        self._transforms = None  # will be set in _execute
        self._prepared_future = None

        # variables from vanilla tornado, not avaiable in rw
        # self.path_args
        # self.path_kwargs
        # self.ui
        self.clear()
        self.request.connection.set_close_callback(self.on_connection_close)
        self.initialize(**kwargs)

    def render(self, template_name, **kwargs):
        """Render..."""
        raise NotImplementedError()

    def render_string(self, template_name, **kwargs):
        """Generate the given template with the given arguments.

        We return the generated byte string (in utf8). To generate and
        write a template as a response, use render() above.
        """
        raise NotImplementedError()

    def get_template_namespace(self):
        """Returns a dictionary to be used as the default template namespace.

        May be overridden by subclasses to add or modify values.

        The results of this method will be combined with additional
        defaults in the `tornado.template` module and keyword arguments
        to `render` or `render_string`.
        """
        raise NotImplementedError()

    def create_template_loader(self, template_path):
        """Returns a new template loader for the given path.

        May be overridden by subclasses.  By default returns a
        directory-based loader on the given path, using the
        ``autoescape`` application setting.  If a ``template_loader``
        application setting is supplied, uses that instead.
        """
        raise NotImplementedError()

    def finish(self, chunk=None):
        """Finishes this response, ending the HTTP request."""
        if self._finished:
            raise RuntimeError("finish() called twice.  May be caused "
                               "by using async operations without the "
                               "@asynchronous decorator.")

        if chunk is not None:
            self.write(chunk)

        # Automatically support ETags and add the Content-Length header if
        # we have not flushed any content yet.
        if not self._headers_written:
            if (self._status_code == 200 and
                self.request.method in ("GET", "HEAD") and
               "Etag" not in self._headers):
                self.set_etag_header()
                if self.check_etag_header():
                    self._write_buffer = []
                    self.set_status(304)
            if self._status_code == 304:
                assert not self._write_buffer, "Cannot send body with 304"
                self._clear_headers_for_304()
            elif "Content-Length" not in self._headers:
                content_length = sum(len(part) for part in self._write_buffer)
                self.set_header("Content-Length", content_length)

        if hasattr(self.request, "connection"):
            # Now that the request is finished, clear the callback we
            # set on the HTTPConnection (which would otherwise prevent the
            # garbage collection of the RequestHandler when there
            # are keepalive connections)
            self.request.connection.set_close_callback(None)

        self.flush(include_footers=True)
        self.request.finish()
        self._log()
        self._finished = True
        self.on_finish()

    # methods to investigate for overwriting
    # def locale(self):
    # def get_user_locale(self):
    # def get_browser_locale(self, default="en_US"):
    # def current_user(self):
    # def current_user(self, value):
    # def get_current_user(self):
    # def _when_complete(self, result, callback):

    @gen.coroutine
    def _execute(self, transforms, *args, **kwargs):
        """Executes this request with the given output transforms."""
        self._transforms = transforms
        try:
            if self.request.method not in self.SUPPORTED_METHODS:
                raise HTTPError(405)
            self.path_args = [self.decode_argument(arg) for arg in args]
            self.path_kwargs = dict((k, self.decode_argument(v, name=k))
                                    for (k, v) in kwargs.items())
            # If XSRF cookies are turned on, reject form submissions without
            # the proper cookie
            if self.request.method not in ("GET", "HEAD", "OPTIONS") and \
                    self.application.settings.get("xsrf_cookies"):
                self.check_xsrf_cookie()

            result = self.prepare()
            if is_future(result):
                result = yield result
            if result is not None:
                raise TypeError("Expected None, got %r" % result)
            if self._prepared_future is not None:
                # Tell the Application we've finished with prepare()
                # and are ready for the body to arrive.
                self._prepared_future.set_result(None)
            if self._finished:
                return

            if _has_stream_request_body(self.__class__):
                # In streaming mode request.body is a Future that signals
                # the body has been completely received.  The Future has no
                # result; the data has been passed to self.data_received
                # instead.
                try:
                    yield self.request.body
                except iostream.StreamClosedError:
                    return

            result = self.handle_request()
            if is_future(result):
                result = yield result
            if result is not None:
                raise TypeError("Expected None, got %r" % result)
            if self._auto_finish and not self._finished:
                self.finish()
        except Exception as e:
            self._handle_request_exception(e)
            if (self._prepared_future is not None and
                    not self._prepared_future.done()):
                # In case we failed before setting _prepared_future, do it
                # now (to unblock the HTTP server).  Note that this is not
                # in a finally block to avoid GC issues prior to Python 3.4.
                self._prepared_future.set_result(None)

    def handle_request(self):
        routing_table = rw.scope.get('rw.http')['routing_table']
        prefix, fn, args = routing_table.find_route(self.request.method, self.request.path)
        current_scope = rw.scope.get_current_scope()
        current_scope['rw.routing.prefix'] = prefix
        current_scope['url_variables'] = args

        if fn is None:
            raise tornado.web.HTTPError(404)

        # only supply arguments if those are "welcome"
        if hasattr(fn, '_rw_wrapped_function'):
            arg_spec = inspect.getargspec(fn._rw_wrapped_function)
        else:
            arg_spec = inspect.getargspec(fn)

        if arg_spec.keywords is not None:
            # fn accepts **keywords arguments so we pass all variables
            return fn(**args)

        call_args = {}
        for arg, value in args.items():
            if arg in arg_spec.args:
                call_args[arg] = value
        return fn(**call_args)

    # overwrite methodes that are not supported to make sure
    # they get not used by accident.
    # TODO: point to alternatives in doc strings

    def get_template_path(self):
        """tornado API, not available in rw"""
        raise NotImplementedError()

    def static_url(self, path, include_host=None, **kwargs):
        """tornado API, not available in rw"""
        raise NotImplementedError()

    def reverse_url(self, name, *args):
        """tornado API, not available in rw"""
        raise NotImplementedError()

    def get_login_url(self):
        """tornado API, not available in rw"""
        raise NotImplementedError()

    def _ui_module(self, name, module):
        """tornado internal method, not used in rw"""
        raise NotImplementedError()

    def _ui_method(self, method):
        """tornado internal method, not used in rw"""
        raise NotImplementedError()



import inspect
import tornado.web

from . import scope

import rw.plugin
import rw.routing
import rw.template


class Module(rw.plugin.Plugin):
    def __init__(self, name, resources=None):
        super(Module, self).__init__(name)
        self.resources = name if resources is None else resources
        self.activate_event.add(self.setup)
        self.routes = []
        self.sub_rt = []
        self.sub_request_handler = []
        self.template_env = None

    def setup(self, top=True):
        # if a Module instance is created inside some python
        # module __init__ we cannot create the template env
        # at module creation as it results in the module
        # itself being imported while getting setup
        routes = rw.routing.RoutingTable(self.name)
        for args in self.routes:
            routes.add_route(*args)

        for path, module, _, _ in self.sub_rt:
            sub_routes = module.setup(False)
            routes.add_child(path, sub_routes)

        for args in self.sub_request_handler:
            routes.add_request_handler(*args)

        if top:
            routes.setup()
            current_scope = scope.get_current_scope()
            if current_scope is not None:
                current_scope.setdefault('rw.http', {})['routing_table'] = routes
        return routes

    @scope.inject
    def render_template(self, template_name, template_env, handler):
        if not template_name.startswith('/'):
            template_name = self.resources + '/' + template_name
        template = template_env.get_template(template_name)
        handler.finish(template.render(**handler))

    def get(self, path):
        """Expose a function for HTTP GET requests

        Example usage::

            @mod.get('/')
            def index(handler):
                ...
        """
        def wrapper(fn):
            fn = scope.inject(fn)
            fn.rw_route = self.routes.append(('get', path, self, fn))
            return fn
        return wrapper

    def post(self, path):
        """Expose a function for HTTP POST requests

        Example usage::

            @post('/save')
            def save(self):
                ...
        """
        def wrapper(fn):
            fn = scope.inject(fn)
            fn.rw_route = self.routes.append(('post', path, self, fn))
            return fn
        return wrapper

    def put(self, path):
        """Expose a function for HTTP PUT requests

        Example usage::

            @put('/elements/<name>')
            def save(self, name):
                ...
        """
        def wrapper(fn):
            fn = scope.inject(fn)
            fn.rw_route = self.routes.append(('put', path, self, fn))
            return fn
        return wrapper

    def delete(self, path):
        """Expose a function for HTTP DELETE requests

        Example usage::

            @delete('/elements/<name>')
            def delete(self, name):
                ...
        """
        def wrapper(fn):
            fn = scope.inject(fn)
            fn.rw_route = self.routes.append(('delete', path, self, fn))
            return fn
        return wrapper

    def options(self, path):
        """Expose a function for HTTP OPTIONS requests

        Example usage::

            @options('/')
            def server_options(self, name):
                ...
        """
        def wrapper(fn):
            fn = scope.inject(fn)
            fn.rw_route = self.routes.append(('options', path, self, fn))
            return fn
        return wrapper

    def mount(self, path, module, handler_args=None, name=None):
        if handler_args is None:
            handler_args = {}
        if inspect.isclass(module) and issubclass(module, tornado.web.RequestHandler):
            self.sub_request_handler.append((path, module, handler_args, name))
        else:
            self.sub_rt.append((path, module, handler_args, name))


def url_for(func, **kwargs):
    if isinstance(func, str):
        if func.startswith('.'):
            # relative to current module
            routing_table = scope.get('rw.http')['routing_table']
            prefix = scope.get('rw.routing.prefix')
            path = (prefix + func).lstrip('.')
            return routing_table.get_path(path, kwargs)
        else:
            # absolute
            routing_table = scope.get('rw.http')['routing_table']
            return routing_table.get_path(func, kwargs)
    else:
        return func.rw_route.get_path(kwargs)



# -*- coding: utf-8 -*-
import os
import sys

from distutils.command.sdist import sdist
from setuptools import setup, find_packages
import setuptools.command.test

BASE_PATH = os.path.abspath(os.path.dirname(__file__))
version_suffix = ''


class TestCommand(setuptools.command.test.test):
    def finalize_options(self):
        setuptools.command.test.test.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        fails = []
        from tox.config import parseconfig
        from tox.session import Session

        config = parseconfig(self.test_args)
        retcode = Session(config).runcommand()
        if retcode != 0:
            fails.append('tox returned errors')

        import pep8
        style_guide = pep8.StyleGuide(config_file=BASE_PATH + '/.pep8')
        style_guide.input_dir(BASE_PATH + '/rw')
        if style_guide.options.report.get_count() != 0:
            fails.append('pep8 returned errros for rw/')

        style_guide = pep8.StyleGuide(config_file=BASE_PATH + '/.pep8')
        style_guide.input_dir(BASE_PATH + '/test')
        if style_guide.options.report.get_count() != 0:
            fails.append('pep8 returned errros for test/')

        if fails:
            print('\n'.join(fails))
            sys.exit(1)


setup(
    name="rueckenwind",
    version="0.4.1",
    url='https://github.com/FlorianLudwig/rueckenwind',
    description='tornado based webframework',
    author='Florian Ludwig',
    author_email='vierzigundzwei@gmail.com',
    install_requires=['tornado>=4.0.0,<5.0',
                      'jinja2',
                      'babel',
                      'argcomplete>=0.6.6,<1.0',
                      'mock',
                      'configobj',
                      'chardet',
                      'pytz',
                      'PyYAML>=3.10',
                      'future'
                      ],
    extras_requires={
        'test': ['tox', 'pytest', 'pep8'],
        'docs': ['sphinx_rtd_theme']
    },
    packages=find_packages(exclude=['*.test', '*.test.*']),
    include_package_data=True,
    package_data={
        'rw': ['*.html', '*.css', 'templates/html5', 'templates/form', 'templates/nginx']
    },
    entry_points={
        'console_scripts': [
            'rw = rw.cli:main',
        ],
    },
    cmdclass={
        'test': TestCommand
    },
    license="http://www.apache.org/licenses/LICENSE-2.0",
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)



# Copyright 2014 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import, division, print_function, with_statement

import rw.gen
import rw.event
import rw.scope


class Plugin(object):
    def __init__(self, name):
        self.name = name
        self.activate_event = rw.event.Event('PLUGIN_{}_ACTIVATE'.format(name))

    def init(self, function):
        function = rw.scope.inject(function)
        self.activate_event.add(function)
        return function

    @rw.scope.inject
    @rw.gen.coroutine
    def activate(self, scope):
        plugins = scope.get('rw.plugins', {})
        if self not in plugins:
            yield self.activate_event()
            plugins[self] = {}


# Copyright 2014 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import tornado.gen

from . import event


PHASE_CONFIGURATION = event.Event('PHASE_CONFIGURATION')
PHASE_SETUP = event.Event('PHASE_SETUP')
PHASE_START = event.Event('PHASE_START')
PHASE_POST_START = event.Event('PHASE_POST_START')


@tornado.gen.coroutine
def start():
    yield PHASE_CONFIGURATION()
    yield PHASE_SETUP()
    yield PHASE_START()
    yield PHASE_POST_START()


# Copyright 2014 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import, division, print_function, with_statement

import sys
import contextlib
import functools
import inspect

from tornado import stack_context

import rw.cfg
from . import gen


NOT_PROVIDED = object()
SCOPE_CHAIN = None


class OutsideScopeError(Exception):
    pass


class Scope(dict):
    def __init__(self, name=None):
        super(Scope, self).__init__()
        self._provider = {}
        self._subscopes = {}
        self.name = name
        self.plugins = set()

    def provider(self, key, provider):
        self._provider[key] = provider

    @gen.coroutine
    def activate(self, plugin):
        yield plugin.activate()
        self.plugins.add(plugin)

    def subscope(self, key):
        if key not in self._subscopes:
            name = '{}.{}'.format(self.name, key)
            subscope = SubScope(name, self)
            self._subscopes[key] = subscope
        return self._subscopes[key]

    def get(self, key, default=NOT_PROVIDED, scopes=None):
        """

        :param str key:
        :param default:
        :param list[Scope] scopes:
        :param str prefix:
        :return: :raise IndexError:
        """
        if scopes is None:
            scopes = list(reversed(SCOPE_CHAIN))
        if key == 'scope':
            return self

        for i, scope in enumerate(scopes):
            if key in scope:
                return scope[key]
            elif key in scope._provider:
                scope[key] = scope._provider[key]()
                del scope._provider[key]
                return scope[key]
            elif key in scope._subscopes:
                return SubScopeView(key, scopes)

        if default is not NOT_PROVIDED:
            return default

        msg = 'No value for "{}" stored and no default given'.format(key)
        raise IndexError(msg)

    def __call__(self):
        return stack_context.StackContext(functools.partial(set_context, self))


class SubScope(Scope):
    def __init__(self, name, parent):
        self.parent = parent
        super(SubScope, self).__init__(name)


class SubScopeView(object):
    def __init__(self, key, scope_chain):
        self.key = key
        self.scope_chain = scope_chain

    def __getitem__(self, item):
        for scope in self.scope_chain:
            if self.key in scope._subscopes:
                if item in scope._subscopes[self.key]:
                    return scope._subscopes[self.key][item]
        raise IndexError()

    def __eq__(self, other):
        return (
            isinstance(other, SubScopeView)
            and self.key == other.key
            and self.scope_chain == other.scope_chain
        )


@contextlib.contextmanager
def set_context(scope):
    global SCOPE_CHAIN
    if SCOPE_CHAIN is None:
        SCOPE_CHAIN = []
    SCOPE_CHAIN.append(scope)
    try:
        yield
    finally:
        # TODO write unit test to get current_scope to be None
        SCOPE_CHAIN.pop()


def get_current_scope():
    return SCOPE_CHAIN[-1] if SCOPE_CHAIN else None


def get(key, default=NOT_PROVIDED):
    if not SCOPE_CHAIN:
        raise OutsideScopeError()
    return SCOPE_CHAIN[-1].get(key, default, list(reversed(SCOPE_CHAIN)))


def inject(fn):
    fn_inspect = getattr(fn, '_rw_wrapped_function', fn)
    arg_spec = inspect.getargspec(fn_inspect)

    @functools.wraps(fn)
    def wrapper(*args, **kwargs):
        if len(args) < len(arg_spec.args):
            # possible injection
            missing_args = set(arg_spec.args[len(args):])
            for key in missing_args:
                if key not in kwargs:
                    if not SCOPE_CHAIN:
                        raise OutsideScopeError('Cannot use inject outside of scope')
                    try:
                        kwargs[key] = get(key)
                    except IndexError:
                        # the key might not be inside scope but there might be
                        # a default parameter defined inside the function
                        pass

        try:
            return fn(*args, **kwargs)
        except:
            msg = 'Error injecting into {}.{}'
            print(msg.format(fn.__module__, fn.__name__), file=sys.stderr)
            raise

    return wrapper


@gen.coroutine
def setup_app_scope(name, scope):
    """Load confing and activate plugins accordingly"""
    settings = rw.cfg.read_configs(name)
    scope['settings'] = settings

    # load plugins
    plugins = []
    for plugin_name, active in settings.get('rw.plugins', {}).items():
        plugin = __import__(plugin_name)
        plugin_path = plugin_name.split('.')[1:] + ['plugin']
        for sub in plugin_path:
            plugin = getattr(plugin, sub)
        plugins.append(scope.activate(plugin))

    yield plugins
    raise rw.gen.Return(settings)
